NONE = 0
GET = 1
LIST = 1 << 1
CREATE = 1 << 2
UPDATE = 1 << 3
DELETE = 1 << 4
QUERIES = GET | LIST
MUTATIONS = CREATE | UPDATE | DELETE
QUERIES_AND_MUTATIONS = QUERIES | MUTATIONS
ALL = (1 << 5) | QUERIES_AND_MUTATIONS
